<?php

namespace Drupal\librejs\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\TableSortExtender;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException;

/**
 * Returns responses for LibreJS routes.
 */
class LibrejsController extends ControllerBase {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('database')
    );
  }

  /**
   * Constructs a LibrejsController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Builds the response.
   *
   * @return mixed[]
   *   The renderable page.
   */
  public function build(): array {

    $build['content'] = [
      '#type' => 'table',
      '#header' => [
        [
          'data' => $this->t('Script'),
          'field' => 'librejs.data',
          'sort' => 'asc',
        ],
        [
          'data' => $this->t('License'),
          'field' => 'librejs.license',
          'class' => [RESPONSIVE_PRIORITY_MEDIUM],
        ],
        [
          'data' => $this->t('Source'),
          'field' => 'librejs.source',
          'class' => [RESPONSIVE_PRIORITY_LOW],
        ],
      ],
      '#attributes' => ['id' => 'jslicense-labels1'],
      '#cache' => [
        'contexts' => ['url.query_args:sort', 'url.query_args:order'],
        'tags' => ['librejs_jslicense'],
      ],
    ];

    $query = $this->database->select('librejs', 'librejs');
    // @fixme Try/catch for multiple versions of core.
    try {
      // @fixme Drupal 9.3 compatibility.
      $query = $query->extend(TableSortExtender::class);
    }
    catch (ServiceNotFoundException $e) {
      $query = $query->extend('table_sort');
    }
    // @fixme Static analysis wants these method_exists() calls.
    if (method_exists($query, 'fields')) {
      $query->fields('librejs', [
        'data',
        'license',
        'url',
        'source',
        'version',
        'type',
      ]);
    }
    if (method_exists($query, 'orderByHeader')) {
      $query->orderByHeader($build['content']['#header']);
    }
    if (method_exists($query, 'execute')) {
      $javascript = $query->execute();
      foreach ($javascript as $js) {
        if ($js->type == 'external') {
          $url = Url::fromUri($js->data);
        }
        else {
          if ($js->version == -1) {
            $query = [$this->state()->get('system.css_js_query_string', '0') => NULL];
          }
          else {
            $query = ['v' => $js->version];
          }
          $url = Url::fromUri("base:{$js->data}", ['query' => $query]);
        }
        $build['content']['#rows'][] = [
          [
            'data' => Link::fromTextAndUrl($js->data, $url),
            'style' => 'overflow-wrap: anywhere',
          ],
          Link::fromTextAndUrl($js->license, Url::fromUri($js->url)),
          [
            'data' => $js->source ? Link::fromTextAndUrl($js->source, Url::fromUri($js->source)) : '',
            'style' => 'overflow-wrap: anywhere',
          ],
        ];
      }
    }
    return $build;
  }

}
